define(["exports", "../../../lit-element/lit-element.js", "../../../lit-element-router/lit-element-router.js", "../../../@polymer/iron-icon/iron-icon.js", "../../../@polymer/iron-icons/iron-icons.js"], function (_exports, _litElement, _litElementRouter, _ironIcon, _ironIcons) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.ElmslnStudioButton = void 0;

  /**
   * Copyright 2020 The Pennsylvania State University
   * @license Apache-2.0, see License.md for full text.
   */

  /**
   * `elmsln-studio-button`
   * Navigation Link for ELMS:LN Studio
   *
   * @customElement elmsln-studio-button
   * @lit-html
   * @lit-element
   */
  class ElmslnStudioButton extends (0, _litElementRouter.navigator)(_litElement.LitElement) {
    /**
     * Store the tag name to make it easier to obtain directly.
     * @notice function name must be here for tooling to operate correctly
     */
    static get tag() {
      return "elmsln-studio-button";
    }

    static get properties() {
      return {
        path: {
          type: String
        },
        icon: {
          type: String
        }
      };
    }

    static get styles() {
      return (0, _litElement.css)`
      :host {
        display: block;
      }
      :host([hidden]) {
        display: none;
      }
      button {
        border: none;
        background-color: transparent;
      }
      button:focus,
      button:hover {
        outline: none;
      }
    `;
    }

    constructor() {
      super();
      this.path = "";
      this.icon = "";
    }

    render() {
      return (0, _litElement.html)`
      <button @click="${this.buttonClick}">
        <iron-icon
          aria-hidden="true"
          ?hidden="${!this.icon}"
          icon="${this.icon}"
        ></iron-icon>
        <slot></slot>
      </button>
    `;
    }

    buttonClick(event) {
      this.navigate(`${this.sourcePath}${this.path}`);
    }

  }

  _exports.ElmslnStudioButton = ElmslnStudioButton;
  customElements.define("elmsln-studio-button", ElmslnStudioButton);
});